//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#include "TMainForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "TDDPixelFormat"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Draw_Library_Install"
#pragma link "TDx_Draw"
#pragma link "TDx_DrawSurface"
#pragma link "TDDCaps"
#pragma link "TDDSCaps"
#pragma link "TDDSurfaceDesc"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
FlipCount=0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Width=780;
Height=580;
Position=poScreenCenter;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Draw1->Created) Dx_Draw1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
Msg("Scanning for Available Devices");

NumDevices = 0;
Dx_Draw1->DDEnumerate(NULL,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateButtonClick(TObject *Sender)
{
Msg("Creating Selected Device");
if (Dx_Draw1->Created)
    Dx_Draw1->Destroy();

int index = DevicesListBox->ItemIndex;
Dx_Draw1->Create( Guids[index] );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyButtonClick(TObject *Sender)
{
Msg("Destroying Active Device");

if (Dx_Draw1->Created)
    {
    Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_NORMAL );
    Dx_Draw1->Destroy();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetModeButtonClick(TObject *Sender)
{
if (ModesListBox->Items->Count>0)
    {
    ShowMessage("Please Note:\n\
You may experience difficulty restoring your display mode.\n\
To return to the original display mode at any time, press ALT-R (Restore) or ALT-X (eXit)" );

    RestoreModeButton->Enabled = true;

    int index = ModesListBox->ItemIndex;
    Dx_Draw1->SetDisplayMode( Widths[index], Heights[index], BitDepths[index], RefreshRates[index], 0 );

    Top=0;
    Left=0;

    RestoreModeButton->Enabled = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RestoreModeButtonClick(TObject *Sender)
{
Timer1->Enabled = false;

Msg("Restoring Display Mode");
Dx_Draw1->RestoreDisplayMode();

// repaint the form, using a trick
int old = Top;
Top=Screen->Height+1;
Top=old;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateBackButtonClick(TObject *Sender)
{
if (PrimarySurface->Created) PrimarySurface->Destroy();

// create a complex primary surface
PrimarySurfaceDesc->Flags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
PrimarySurfaceDesc->BackBufferCount = 1;
PrimarySurfaceCaps->Caps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_COMPLEX | DDSCAPS_FLIP;
PrimarySurfaceDesc->SCaps = PrimarySurfaceCaps;

PrimarySurface->Create( PrimarySurfaceDesc, Dx_Draw1 );

// get it's attached backbuffer
if (PrimarySurface->GetAttachedSurface( BackBufferCaps, BackBuffer ))
    // call the backbuffer::OnCreate() event manually.
    BackBuffer->OnCreate(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FlipButtonClick(TObject *Sender)
{
ShowMessage("Note:\n\
During page flipping this form will be erased but is still active.\n\
Press Alt-R to stop the animation and repaint the form." );

RestoreModeButton->Enabled = true;
Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroySurfacesButtonClick(TObject *Sender)
{
if (PrimarySurface->Created) PrimarySurface->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ModesListBoxClick(TObject *Sender)
{
if (ModesListBox->Items->Count>0)
    {
    Msg("Mode Selected");
    SetModeButton->Enabled = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (DevicesListBox->Items->Count>0)
    {
    int index = DevicesListBox->ItemIndex;
    AnsiString device_name = DevicesListBox->Items->Strings[index];
    Msg("Device Selected: '"+device_name+"'");

    CreateButton->Enabled = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Create(TObject *Sender)
{
Msg("Device has been created");

Msg("Setting Fullscreen/Exclusive Mode");
Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE );

Msg("Enumerating Display Modes");
ModesListBox->Items->Clear();
NumModes = 0;
Dx_Draw1->EnumDisplayModes(0,NULL,NULL);

CreateButton->Enabled = false;
DestroyButton->Enabled = true;
CreateBackButton->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Destroy(TObject *Sender)
{
Msg("Device is being Destroyed");
if (PrimarySurface->Created) PrimarySurface->Destroy();

CreateButton->Enabled = true;
DestroyButton->Enabled = false;
CreateBackButton->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1DDEnumerate(TObject *Sender, GUID *Guid,
    AnsiString DriverDescription, AnsiString DriverName, void *Context,
    HMONITOR HMonitor, bool &Finished)
{
if (NumDevices<5)
    {
    Guids[NumDevices] = Guid;
    Names[NumDevices] = DriverName;
    Descriptions[NumDevices] = DriverDescription;
    NumDevices++;

    Msg("Found a device: "+DriverName+", "+DriverDescription);
    DevicesListBox->Items->Add( DriverName+", "+DriverDescription );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1EnumDisplayModes(TObject *Sender,
    TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished)
{
int width = SurfaceDesc->Width;
int height = SurfaceDesc->Height;
int refresh_rate = SurfaceDesc->RefreshRate;
int bit_depth;

if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED1) bit_depth = 1;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED2) bit_depth = 2;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED4) bit_depth = 4;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXEDTO8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHA) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPIXELS) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPREMULT) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPDUDV) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPLUMINANCE) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_RGB) bit_depth = SurfaceDesc->PixelFormat->RGBBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_YUV) bit_depth = SurfaceDesc->PixelFormat->YUVBitCount[0];

if (NumModes<50)
    {
    Widths[NumModes] = width;
    Heights[NumModes] = height;
    BitDepths[NumModes] = bit_depth;
    RefreshRates[NumModes] = refresh_rate;
    NumModes++;
    ModesListBox->Items->Add( IntToStr(width)+"x"+IntToStr(height)+"x"+IntToStr(bit_depth)+" @ "+IntToStr(refresh_rate)+"Hz" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PrimarySurfaceCreate(TObject *Sender)
{
Msg("Primary surface has been Created");
DestroySurfacesButton->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PrimarySurfaceDestroy(TObject *Sender)
{
if (BackBuffer->Created) BackBuffer->Destroy();
Msg("Primary Surface is being Destroyed");
DestroySurfacesButton->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BackBufferCreate(TObject *Sender)
{
Msg("Back Buffer has been Created");
DestroySurfacesButton->Enabled = true;
FlipButton->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BackBufferDestroy(TObject *Sender)
{
Msg("Back Buffer is being Destroyed");
FlipButton->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

// setup for animation
bool odd = (++FlipCount%2==0);

// do some animation
HDC hdc;
if (BackBuffer->GetDC(&hdc))
    {
    TCanvas* canvas = new TCanvas();
    if (canvas)
        {
        try
            {
            canvas->Handle = hdc;
            TRect r = canvas->ClipRect;
            int width = r.Right - r.Left;

            canvas->Brush->Style = bsSolid;
            canvas->Brush->Color = clBlack;
            canvas->FillRect( r );

            if (odd) canvas->Brush->Style = bsCross;
            else     canvas->Brush->Style = bsDiagCross;

            canvas->Brush->Color = clGreen;
            canvas->Pen->Style = psSolid;
            canvas->Pen->Color = clRed;
            canvas->Pen->Width = 1;

            canvas->Rectangle( r.Left, r.Top, r.Right, r.Bottom );

            canvas->Font->Size = 48;
            canvas->Font->Style = TFontStyles() << fsBold;

            canvas->Brush->Color = (TColor) RGB(0,0,FlipCount%255);
            canvas->Font->Color = (TColor) RGB(255,FlipCount%255,0);

            if (odd) canvas->TextOut( FlipCount%width, 0, "Odd" );
            else     canvas->TextOut( FlipCount%width, 100, "Even" );

            }
        catch (Exception &e)
            {
            }
        delete canvas;
        }
    BackBuffer->ReleaseDC(hdc);
    }
PrimarySurface->Flip( BackBuffer, DDFLIP_DONOTWAIT );
}
//---------------------------------------------------------------------------
